@@ -84,8 +84,6 @@ enum
 #define CONN_IN_ERROR_BEFORE_IDLE	(1L<<3)
 
 /* AutoCommit functions */
-#define CC_set_autocommit_off(x)	(x->transact_status &= ~CONN_IN_AUTOCOMMIT)
-#define CC_set_autocommit_on(x)		(x->transact_status |= CONN_IN_AUTOCOMMIT)
 #define CC_is_in_autocommit(x)		(x->transact_status & CONN_IN_AUTOCOMMIT)
 
 /* Transaction in/not functions */
@@ -127,11 +125,13 @@ do { \
 } while (0)
 #define CC_REALLOC_return_with_error(t, tp, s, x, m, ret) \
 do { \
-	if (t = (tp *) realloc(t, s), NULL == t) \
+	tp *tmp; \
+	if (tmp = (tp *) realloc(t, s), NULL == tmp) \
 	{ \
 		CC_set_error(x, CONN_NO_MEMORY_ERROR, m, ""); \
 		return ret; \
 	} \
+	t = tmp; \
 } while (0)
 
 /* For Multi-thread */
@@ -305,9 +305,10 @@ typedef struct
 	signed char	bde_environment;
 	signed char	fake_mss;
 	signed char	cvt_null_date_string;
+	signed char	autocommit_public;
+	signed char	accessible_only;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	signed char	xa_opt;
-	signed char	autocommit_normal;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 	GLOBAL_VALUES drivers;		/* moved from driver's option */
 } ConnInfo;
@@ -471,6 +472,7 @@ struct ConnectionClass_
 #define CC_is_onlyread(x)			(x->connInfo.onlyread[0] == '1')
 #define CC_get_escape(x)			(x->escape_in_literal)
 #define CC_fake_mss(x)	(/* 0 != (x)->ms_jet && */ 0 < (x)->connInfo.fake_mss)
+#define CC_accessible_only(x)	(0 < (x)->connInfo.accessible_only && PG_VERSION_GE((x), 7.2))
 #define CC_default_is_c(x)	(CC_is_in_ansi_app(x) || x->ms_jet /* not only */ || TRUE /* but for any other ? */)
 /*	for CC_DSN_info */
 #define CONN_DONT_OVERWRITE		0
@@ -486,6 +488,7 @@ char		CC_cleanup(ConnectionClass *self);
 char		CC_begin(ConnectionClass *self);
 char		CC_commit(ConnectionClass *self);
 char		CC_abort(ConnectionClass *self);
+char		CC_set_autocommit(ConnectionClass *self, BOOL on);
 int		CC_set_translation(ConnectionClass *self);
 char		CC_connect(ConnectionClass *self, char password_req, char *salt);
 char		CC_add_statement(ConnectionClass *self, StatementClass *stmt);
@@ -498,7 +501,8 @@ char		CC_remove_descriptor(ConnectionClass *self, DescriptorClass *desc);
 void		CC_set_error(ConnectionClass *self, int number, const char *message, const char *func);
 void		CC_set_errormsg(ConnectionClass *self, const char *message);
 char		CC_get_error(ConnectionClass *self, int *number, char **message);
-QResultClass *CC_send_query(ConnectionClass *self, char *query, QueryInfo *qi, UDWORD flag, StatementClass *stmt);
+QResultClass *CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UDWORD flag, StatementClass *stmt, const char *appendq);
+#define CC_send_query(self, query, qi, flag, stmt) CC_send_query_append(self, query, qi, flag, stmt, NULL)
 void		CC_clear_error(ConnectionClass *self);
 int		CC_send_function(ConnectionClass *conn, int fnid, void *result_buf, int *actual_result_len, int result_is_int, LO_ARG *argv, int nargs);
 char		CC_send_settings(ConnectionClass *self);
@@ -537,7 +541,8 @@ enum {
 	IGNORE_ABORT_ON_CONN	= 1L /* not set the error result even when  */
 	,CREATE_KEYSET		= (1L << 1) /* create keyset for updatable curosrs */
 	,GO_INTO_TRANSACTION	= (1L << 2) /* issue begin in advance */
-	, ROLLBACK_ON_ERROR	= (1L << 3) /* rollback the query when an error occurs */
+	,ROLLBACK_ON_ERROR	= (1L << 3) /* rollback the query when an error occurs */
+	,END_WITH_COMMIT	= (1L << 4) /* the query ends with COMMMIT command */
 };
 /* CC_on_abort options */
 #define	NO_TRANS		1L
