@@ -57,11 +57,13 @@ static pg_CS CS_Table[] =
 	{ "WIN1255",	WIN1255 },	/* Hebrew since 8.2 */
 	{ "WIN1257",	WIN1257 },	/* Baltic(North Europe) since 8.2 */
 
+	{ "EUC_JIS_2004", EUC_JIS_2004},	/* EUC for SHIFT-JIS-2004 Japanese, since 8.3 */
 	{ "SJIS",	SJIS },
 	{ "BIG5",	BIG5 },
 	{ "GBK",	GBK },		/* since 7.3 */
 	{ "UHC",	UHC },		/* since 7.3 */	
 	{ "GB18030",	GB18030 },	/* since 7.3 */
+	{ "SHIFT_JIS_2004", SHIFT_JIS_2004 },	/* SHIFT-JIS-2004 Japanese, standard JIS X 0213, since 8.3 */
 	{ "OTHER",	OTHER }
 };
 
@@ -206,9 +208,11 @@ pg_mb_maxlen(characterset_code)
 			return 6;
 		case EUC_TW:
 			return 4;
+		case EUC_JIS_2004:
 		case EUC_JP:
 		case GB18030:
 			return 3;
+		case SHIFT_JIS_2004:
 		case SJIS:
 		case BIG5:
 		case GBK:
@@ -252,6 +256,24 @@ pg_CS_stat(int stat,unsigned int character,int characterset_code)
 					stat=0;
 			}
 			break;
+/* SHIFT_JIS_2004 Support. */
+			case SHIFT_JIS_2004:
+			{
+				if (stat < 2 &&
+					character >= 0x81 && character <= 0x9f)
+					stat = 2;
+				else if (stat < 2 &&
+					character >= 0xe0 && character <= 0xef)
+					stat = 2;
+				else if (stat < 2 &&
+					character >= 0xf0 && character <= 0xfc)
+					stat = 2;
+				else if (stat == 2)
+					stat = 1;
+				else
+					stat = 0;
+			}
+			break;
 /* Shift-JIS Support. */
 			case SJIS:
 			{
@@ -304,8 +326,14 @@ pg_CS_stat(int stat,unsigned int character,int characterset_code)
 			}
 			break;
 
-/* EUC_JP Support */
+		case EUC_JIS_2004:
+			/* 0x8f is JIS X 0212 + JIS X 0213(2) 3 byte */
+			/* 0x8e is JIS X 0201 2 byte */
+			/* 0xa0-0xff is JIS X 0213(1) 2 byte */
 		case EUC_JP:
+			/* 0x8f is JIS X 0212 3 byte */
+			/* 0x8e is JIS X 0201 2 byte */
+			/* 0xa0-0xff is JIS X 0208 2 byte */
 			{
 				if (stat < 3 && 
 					character == 0x8f)	/* JIS X 0212 */
