@@ -19,6 +19,7 @@
  */
 
 #include "psqlodbc.h"
+#include "misc.h"
 
 #if (ODBCVER >= 0x0300)
 #include <stdio.h>
@@ -472,14 +473,14 @@ SQLSetEnvAttr(HENV EnvironmentHandle,
 			ret = SQL_SUCCESS;
 			break;
 		case SQL_ATTR_ODBC_VERSION:
-			if ((SQLUINTEGER) Value == SQL_OV_ODBC2)
+			if (SQL_OV_ODBC2 == CAST_UPTR(SQLUINTEGER, Value))
 				EN_set_odbc2(env);
 			else
 				EN_set_odbc3(env);
 			ret = SQL_SUCCESS;
 			break;
 		case SQL_ATTR_OUTPUT_NTS:
-			if ((SQLUINTEGER) Value == SQL_TRUE)
+			if (SQL_TRUE == CAST_UPTR(SQLUINTEGER, Value))
 				ret = SQL_SUCCESS;
 			else
 				ret = SQL_SUCCESS_WITH_INFO;
@@ -635,7 +636,7 @@ inolog("lie=%d\n", ci->drivers.lie);
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSETENVATTR);		/* 1019 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLSETSTMTATTR);	/* 1020 */
 	SQL_FUNC_ESET(pfExists, SQL_API_SQLFETCHSCROLL);	/* 1021 */
-	if (0 != ci->updatable_cursors)
+	if (0 != (ALLOW_BULK_OPERATIONS & ci->updatable_cursors))
 		SQL_FUNC_ESET(pfExists, SQL_API_SQLBULKOPERATIONS);	/* 24 */
 
 	return SQL_SUCCESS;
