@@ -230,10 +230,24 @@ set_statement_option(ConnectionClass *conn,
 			break;
 
 		case 1204: /* SQL_COPT_SS_PRESERVE_CURSORS ? */
+			if (stmt)
+			{
+				SC_set_error(stmt, STMT_OPTION_NOT_FOR_THE_DRIVER, "The option may be for MS SQL Server(Set)", func);
+			}
+			else if (conn)
+			{
+				CC_set_error(conn, CONN_OPTION_NOT_FOR_THE_DRIVER, "The option may be for MS SQL Server(Set)", func);
+			}
+			return SQL_ERROR;
 		case 1227: /* SQL_SOPT_SS_HIDDEN_COLUMNS ? */
 		case 1228: /* SQL_SOPT_SS_NOBROWSETABLE ? */
 			if (stmt)
 			{
+#ifndef	NOT_USED
+				if (0 != vParam)
+					changed = TRUE;
+				break;
+#endif /* NOT_USED */
 				SC_set_error(stmt, STMT_OPTION_NOT_FOR_THE_DRIVER, "The option may be for MS SQL Server(Set)", func);
 			}
 			else if (conn)
@@ -288,8 +302,10 @@ PGAPI_SetConnectOption(
 {
 	CSTR func = "PGAPI_SetConnectOption";
 	ConnectionClass *conn = (ConnectionClass *) hdbc;
+	ConnInfo   *ci = &(conn->connInfo);
 	char		changed = FALSE;
 	RETCODE		retval;
+	BOOL		autocomm_on;
 
 	mylog("%s: entering fOption = %d vParam = %d\n", func, fOption, vParam);
 	if (!conn)
@@ -351,29 +367,33 @@ PGAPI_SetConnectOption(
 			break;
 
 		case SQL_AUTOCOMMIT:
-			if (vParam == SQL_AUTOCOMMIT_ON && CC_is_in_autocommit(conn))
-				break;
-			else if (vParam == SQL_AUTOCOMMIT_OFF && !CC_is_in_autocommit(conn))
-				break;
-			if (CC_is_in_trans(conn))
-				CC_commit(conn);
-
-			mylog("PGAPI_SetConnectOption: AUTOCOMMIT: transact_status=%d, vparam=%d\n", conn->transact_status, vParam);
-
 			switch (vParam)
 			{
-				case SQL_AUTOCOMMIT_OFF:
-					CC_set_autocommit_off(conn);
-					break;
-
 				case SQL_AUTOCOMMIT_ON:
-					CC_set_autocommit_on(conn);
+					autocomm_on = TRUE;
+					break;
+				case SQL_AUTOCOMMIT_OFF:
+					autocomm_on = FALSE;
 					break;
-
 				default:
 					CC_set_error(conn, CONN_INVALID_ARGUMENT_NO, "Illegal parameter value for SQL_AUTOCOMMIT", func);
 					return SQL_ERROR;
 			}
+			if (autocomm_on && SQL_AUTOCOMMIT_OFF != ci->autocommit_public)
+				break;
+			else if (!autocomm_on && SQL_AUTOCOMMIT_OFF == ci->autocommit_public)
+				break;
+			ci->autocommit_public = (autocomm_on ? SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF);
+			mylog("%s: AUTOCOMMIT: transact_status=%d, vparam=%d\n", func, conn->transact_status, vParam);
+
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+			if (NULL != conn->asdum)
+			{
+				mylog("%s: Ignored AUTOCOMMIT in a distributed transaction, OK ?");
+				break;
+			}
+#endif	/* _HANDLE_ENLIST_IN_DTC_ */
+			CC_set_autocommit(conn, autocomm_on);
 			break;
 
 		case SQL_CURRENT_QUALIFIER:		/* ignored */
@@ -522,8 +542,7 @@ PGAPI_GetConnectOption(
 			break;
 
 		case SQL_AUTOCOMMIT:
-			*((SQLUINTEGER *) pvParam) = (SQLUINTEGER) (CC_is_in_autocommit(conn) ?
-								 SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF);
+			*((SQLUINTEGER *) pvParam) = ci->autocommit_public;
 			break;
 
 		case SQL_CURRENT_QUALIFIER:		/* don't use qualifiers */
@@ -770,6 +789,10 @@ PGAPI_GetStmtOption(
 		case SQL_USE_BOOKMARKS:
 			*((SQLINTEGER *) pvParam) = stmt->options.use_bookmarks;
 			break;
+		case 1227: /* SQL_SOPT_SS_HIDDEN_COLUMNS ? */
+		case 1228: /* SQL_SOPT_SS_NOBROWSETABLE ? */
+			*((SQLINTEGER *) pvParam) = 0;
+			break;
 
 		default:
 			{
