@@ -78,7 +78,7 @@ SOCK_Constructor(const ConnectionClass *conn)
 		rv->buffer_filled_out = 0;
 		rv->buffer_read_in = 0;
 
-		if (rv)
+		if (conn)
 			rv->buffer_size = conn->connInfo.drivers.socket_buffersize;
 		else
 			rv->buffer_size = globals.socket_buffersize;
@@ -116,8 +116,7 @@ SOCK_Destructor(SocketClass *self)
 		{
 			if (self->via_libpq)
 			{
-				if (self->pqconn)
-					PQfinish(self->pqconn);
+				PQfinish(self->pqconn);
 				/* UnloadDelayLoadedDLLs(NULL != self->ssl); */
 			}
 			self->via_libpq = FALSE;
@@ -223,9 +222,13 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 		getnameinfo_ptr = (getnameinfo_func)GetProcAddress(ws2_hnd, "getnameinfo"); 
 #endif
 	/*
-	 * If it is a valid IP address, use it. Otherwise use hostname lookup.
+	 * Hostname lookup.
 	 */
-	if (hostname && hostname[0])
+	if (hostname && hostname[0]
+#ifndef	WIN32
+	    && '/' != hostname[0]
+#endif /* WIN32 */
+	   )
 	{
 		char	portstr[16];
 		int	ret;
@@ -251,8 +254,8 @@ SOCK_connect_to(SocketClass *self, unsigned short port, char *hostname, long tim
 	{
 		struct sockaddr_un *un = (struct sockaddr_un *) &(self->sadr_area);
 		family = un->sun_family = AF_UNIX;
-		/* passing NULL means that this only supports the pg default "/tmp" */
-		UNIXSOCK_PATH(un, port, ((char *) NULL));
+		/* passing NULL or '' means pg default "/tmp" */
+		UNIXSOCK_PATH(un, port, hostname);
 		self->sadr_len = UNIXSOCK_LEN(un);
 	}
 #else
@@ -449,6 +452,8 @@ static int SOCK_wait_for_ready(SocketClass *sock, BOOL output, int retry_count)
 	}
 	return ret;
 }
+
+#ifdef USE_SSL
 /*
  *	The stuff for SSL.
  */
@@ -552,6 +557,7 @@ inolog("%s: %d get_error=%d Lasterror=%d\n", func,  n, err, gerrno);
 	return n;
 }
 
+#endif /* USE_SSL */
 
 int
 SOCK_get_id(SocketClass *self)
@@ -727,9 +733,11 @@ SOCK_flush_output(SocketClass *self)
 		return -1;
 	while (self->buffer_filled_out > 0)
 	{
+#ifdef USE_SSL 
 		if (self->ssl)
 			written = SOCK_SSL_send(self, (char *) self->buffer_out + pos, self->buffer_filled_out);
 		else
+#endif /* USE_SSL */
 			written = send(self->socket, (char *) self->buffer_out + pos, self->buffer_filled_out, 0);
 		gerrno = SOCK_ERRNO;
 		if (written < 0)
@@ -773,9 +781,11 @@ SOCK_get_next_byte(SocketClass *self)
 		 */
 		self->buffer_read_in = 0;
 retry:
+#ifdef USE_SSL 
 		if (self->ssl)
 			self->buffer_filled_in = SOCK_SSL_recv(self, (char *) self->buffer_in, self->buffer_size);
 		else
+#endif /* USE_SSL */
 			self->buffer_filled_in = recv(self->socket, (char *) self->buffer_in, self->buffer_size, 0);
 		gerrno = SOCK_ERRNO;
 
@@ -846,9 +856,11 @@ SOCK_put_next_byte(SocketClass *self, UCHAR next_byte)
 		/* buffer is full, so write it out */
 		do
 		{
+#ifdef USE_SSL
 			if (self->ssl)
 				bytes_sent = SOCK_SSL_send(self, (char *) self->buffer_out + pos, self->buffer_filled_out);
 			else
+#endif /* USE_SSL */
 				bytes_sent = send(self->socket, (char *) self->buffer_out + pos, self->buffer_filled_out, 0);
 			gerrno = SOCK_ERRNO;
 			if (bytes_sent < 0)
