@@ -343,11 +343,13 @@ do { \
 } while (0)
 #define	SC_REALLOC_return_with_error(t, tp, s, a, m, r) \
 do { \
-	if (t = (tp *) realloc(t, s), NULL == t) \
+	tp *tmp; \
+	if (tmp = (tp *) realloc(t, s), NULL == tmp) \
 	{ \
 		SC_set_error(a, STMT_NO_MEMORY_ERROR, m, "SC_REALLOC"); \
 		return r; \
 	} \
+	t = tmp; \
 } while (0)
 
 /*	options for SC_free_params() */
@@ -439,6 +441,7 @@ RETCODE		SC_initialize_and_recycle(StatementClass *self);
 void		SC_initialize_cols_info(StatementClass *self, BOOL DCdestroy, BOOL parseReset);
 int		statement_type(const char *statement);
 char		parse_statement(StatementClass *stmt, BOOL);
+SQLRETURN	SC_set_SS_columnkey(StatementClass *stmt);
 Int4		SC_pre_execute(StatementClass *self);
 char		SC_unbind_cols(StatementClass *self);
 char		SC_recycle_statement(StatementClass *self);
